##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Viewlet tests
"""
import doctest
import itertools
import re
import unittest

import zope.component.testing
from zope.configuration import xmlconfig

from zope.testing import renormalizing

import z3c.macro.tales
import z3c.macro.zcml


def setUp(test):
    zope.component.testing.setUp(test)
    #root = setup.placefulSetUp(site=True)
    #test.globs['root'] = root


def setUpZPT(test):
    setUp(test)
    from zope.browserpage import metaconfigure
    metaconfigure.registerType('macro', z3c.macro.tales.MacroExpression)


def setUpZ3CPT(suite):
    setUp(suite)
    import z3c.pt
    import z3c.ptcompat
    xmlconfig.XMLConfig('configure.zcml', z3c.pt)()
    xmlconfig.XMLConfig('configure.zcml', z3c.ptcompat)()


def tearDown(test):
    zope.component.testing.tearDown(test)


checker = renormalizing.RENormalizing([
    (re.compile("u('.*?')"), r"\1"),
    (re.compile('u(".*?")'), r"\1"),
    # Python 3 adds module name to exceptions;
    # The output of this one is too complex for IGNORE_EXCEPTION_MODULE_IN_PYTHON2
    (re.compile('zope.configuration.xmlconfig.ZopeXMLConfigurationError'),
     'ZopeXMLConfigurationError'),
])

def test_suite():
    tests = ((
        doctest.DocFileSuite(
            'README.rst',
            setUp=setUp, tearDown=tearDown,
            checker=checker,
            optionflags=(doctest.NORMALIZE_WHITESPACE
                         | doctest.ELLIPSIS
                         | renormalizing.IGNORE_EXCEPTION_MODULE_IN_PYTHON2),
        ),
        doctest.DocFileSuite(
            'zcml.rst',
            setUp=setUp, tearDown=tearDown,
            checker=checker,
            optionflags=(doctest.NORMALIZE_WHITESPACE
                         | doctest.ELLIPSIS
                         | renormalizing.IGNORE_EXCEPTION_MODULE_IN_PYTHON2),
        )
    ) for setUp in (setUpZ3CPT, setUpZPT))

    return unittest.TestSuite(itertools.chain(*tests))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
z3c.macro package.

Importing this package has the side-effect of registering the 'macro' expression
type in Chameleon (if z3c.pt is installed)
"""

try:
    # register chameleon ``macro`` tales expression for BaseTemplate
    # there is not adapter or other registration support built in in
    # z3c.pt and apply our tales expression to any page template or
    # offer a custom PageTemplate is no option
    from z3c.macro import tales
    import z3c.pt.pagetemplate
    z3c.pt.pagetemplate.BaseTemplate.expression_types['macro'] = tales.MacroExpr
except ImportError: # pragma: no cover
    # we do not support z3c.pt
    pass



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Interfaces for z3c.macro.
"""

from zope.tales import interfaces
from zope.pagetemplate.interfaces import IPageTemplate


class IMacroTemplate(IPageTemplate):
    """A macro template."""


class IMacroExpression(interfaces.ITALESExpression):
    """Return the HTML content of the named provider.

    To call a macro from a page template use the the following syntax::

      <metal:block use-macro="macro:macroname">
        <metal:block fill-slot="slotname">
          this content get rendered in the defined slot of the macro
        <\metal:block>
      <\metal:block>

      or

      <metal:block use-macro="macro:macroname">
        this content get replaced by the defined macro
      </metal:block>

    The ``macro:`` TALES expression calles a named adapter adapting
    (context, request) or (context, request, view), depending on the usage
    of the view attribute in the macro directive. A macro provides the
    interface IMacroTemplate.
    """



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""ZCML Meta-Directives
"""
import os

import zope.interface
import zope.schema
import zope.configuration.fields
from zope.configuration.exceptions import ConfigurationError
from zope.component import zcml
from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile

from z3c.macro import interfaces


class IMacroDirective(zope.interface.Interface):
    """Parameters for the template directive."""

    template = zope.configuration.fields.Path(
        title=u'Template defining a named macro.',
        description=u"""Refers to a file containing a page template
            (should end in extension ``.pt`` or ``.html``).
            """,
        required=True,
        )

    name = zope.schema.TextLine(
        title=u'Name',
        description=u"""
            The macro name which this macro is registered for. The macro
            name can be the same defined in metal:define-macro but does
            not have to be the same. If no macro attribute is given the
            name is used as the name defined in metal:define-macro. If you
            need to register a macro under a different name as the defined
            one, you can use the macro attribute which have to reference
            the metal.define-macro name. The TALES expression calls macros
            by this name and returns the macro within the same name or with
            the name defined in the macro attribute.
            """,
        required=True,
        default=u'',
        )

    macro = zope.schema.TextLine(
        title=u'Macro',
        description=u"""
            The name of the macro to be used. This allows us to reference
            the named  macro defined with metal:define-macro if we use a
            different IMacroDirective name.
            """,
        required=False,
        default=u'',
        )

    for_ = zope.configuration.fields.GlobalObject(
        title=u'Context',
        description=u'The context for which the macro should be used',
        required=False,
        default=zope.interface.Interface,
        )

    view = zope.configuration.fields.GlobalObject(
        title=u'View',
        description=u'The view for which the macro should be used',
        required=False,
        default=IBrowserView)

    layer = zope.configuration.fields.GlobalObject(
        title=u'Layer',
        description=u'The layer for which the macro should be used',
        required=False,
        default=IDefaultBrowserLayer,
        )

    contentType = zope.schema.ASCIILine(
        title=u'Content Type',
        description=u'The content type identifies the type of data.',
        default='text/html',
        required=False,
        )


class MacroFactory(object):
    """Macro factory."""

    def __init__(self, path, macro, contentType):
        self.path = path
        self.macro = macro
        self.contentType = contentType

    def __call__(self, context, view, request):
        template = ViewPageTemplateFile(self.path,
                                        content_type=self.contentType)
        return template.macros[self.macro]


def registerMacroFactory(_context, path, name, macro, for_, view, layer,
                         contentType):
    """Register a named macro factory adapter."""

    factory = MacroFactory(path, macro, contentType)

    # register the macro
    zcml.adapter(_context, (factory,), interfaces.IMacroTemplate,
                 (for_, view, layer), name=name)


def macroDirective(_context, template, name, macro=u'',
                   for_=zope.interface.Interface, view=IBrowserView,
                   layer=IDefaultBrowserLayer, contentType='text/html'):

    # Make sure that the template exists
    path = os.path.abspath(str(_context.path(template)))
    if not os.path.isfile(path):
        raise ConfigurationError("No such file", template)

    if not macro:
        macro = name

    registerMacroFactory(_context, path, name, macro, for_, view, layer,
                         contentType)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Provider tales expression registrations


"""
__docformat__ = 'restructuredtext'


import zope.component
import zope.interface
from zope.tales import expressions

from z3c.macro import interfaces

def get_macro_template(context, view, request, name):
    return zope.component.getMultiAdapter(
        (context, view, request), interface=interfaces.IMacroTemplate, name=name)

@zope.interface.implementer(interfaces.IMacroExpression)
class MacroExpression(expressions.StringExpr):
    """Collect named IMacroTemplate via a TAL namespace called ``macro``."""

    def __call__(self, econtext):
        name = super(MacroExpression, self).__call__(econtext)
        context = econtext.vars['context']
        request = econtext.vars['request']
        view = econtext.vars['view']
        return get_macro_template(context, view, request, name)

try:
    # define chameleon  ``macro`` expression

    from chameleon.tales import StringExpr
    from chameleon.astutil import Static
    from chameleon.astutil import Symbol
    from chameleon.codegen import template

    class MacroGetter(object):
        """Collect named IMacroTemplate via a TAL namespace called ``macro``."""

        def __call__(self, context, request, view, name):
            return zope.component.getMultiAdapter(
                (context, view, request), interface=interfaces.IMacroTemplate,
                name=name)

    class MacroExpr(StringExpr):
        traverser = Static(
            template("cls()", cls=Symbol(MacroGetter), mode="eval")
            )

        def __call__(self, target, engine):
            assignment = super(MacroExpr, self).__call__(target, engine)

            return assignment + template(
                "target = traverse(context, request, view, target.strip())",
                target=target,
                traverse=self.traverser,
            )

except ImportError: # pragma: no cover
    pass
